@@ -20,7 +20,7 @@
 #
 import logging
 import random
-from typing import TYPE_CHECKING, Dict, Iterable, List, Optional, Set, Tuple
+from typing import TYPE_CHECKING, Iterable, Optional
 
 import attr
 
@@ -28,7 +28,6 @@ from synapse.api.constants import EduTypes
 from synapse.api.errors import AuthError, ShadowBanError, SynapseError
 from synapse.appservice import ApplicationService
 from synapse.metrics.background_process_metrics import (
-    run_as_background_process,
     wrap_as_background_process,
 )
 from synapse.replication.tcp.streams import TypingStream
@@ -78,9 +77,10 @@ class FollowerTypingHandler:
     """
 
     def __init__(self, hs: "HomeServer"):
+        self.hs = hs  # nb must be called this for @wrap_as_background_process
         self.store = hs.get_datastores().main
         self._storage_controllers = hs.get_storage_controllers()
-        self.server_name = hs.config.server.server_name
+        self.server_name = hs.hostname
         self.clock = hs.get_clock()
         self.is_mine_id = hs.is_mine_id
         self.is_mine_server_name = hs.is_mine_server_name
@@ -96,15 +96,15 @@ class FollowerTypingHandler:
             )
 
         # map room IDs to serial numbers
-        self._room_serials: Dict[str, int] = {}
+        self._room_serials: dict[str, int] = {}
         # map room IDs to sets of users currently typing
-        self._room_typing: Dict[str, Set[str]] = {}
+        self._room_typing: dict[str, set[str]] = {}
 
-        self._member_last_federation_poke: Dict[RoomMember, int] = {}
+        self._member_last_federation_poke: dict[RoomMember, int] = {}
         self.wheel_timer: WheelTimer[RoomMember] = WheelTimer(bucket_size=5000)
         self._latest_room_serial = 0
 
-        self._rooms_updated: Set[str] = set()
+        self._rooms_updated: set[str] = set()
 
         self.clock.looping_call(self._handle_timeouts, 5000)
         self.clock.looping_call(self._prune_old_typing, FORGET_TIMEOUT)
@@ -142,8 +142,11 @@ class FollowerTypingHandler:
         if self.federation and self.is_mine_id(member.user_id):
             last_fed_poke = self._member_last_federation_poke.get(member, None)
             if not last_fed_poke or last_fed_poke + FEDERATION_PING_INTERVAL <= now:
-                run_as_background_process(
-                    "typing._push_remote", self._push_remote, member=member, typing=True
+                self.hs.run_as_background_process(
+                    "typing._push_remote",
+                    self._push_remote,
+                    member=member,
+                    typing=True,
                 )
 
         # Add a paranoia timer to ensure that we always have a timer for
@@ -192,7 +195,7 @@ class FollowerTypingHandler:
             logger.exception("Error pushing typing notif to remotes")
 
     def process_replication_rows(
-        self, token: int, rows: List[TypingStream.TypingStreamRow]
+        self, token: int, rows: list[TypingStream.TypingStreamRow]
     ) -> None:
         """Should be called whenever we receive updates for typing stream."""
 
@@ -214,7 +217,7 @@ class FollowerTypingHandler:
             self._rooms_updated.add(row.room_id)
 
             if self.federation:
-                run_as_background_process(
+                self.hs.run_as_background_process(
                     "_send_changes_in_typing_to_remotes",
                     self._send_changes_in_typing_to_remotes,
                     row.room_id,
@@ -223,7 +226,7 @@ class FollowerTypingHandler:
                 )
 
     async def _send_changes_in_typing_to_remotes(
-        self, room_id: str, prev_typing: Set[str], now_typing: Set[str]
+        self, room_id: str, prev_typing: set[str], now_typing: set[str]
     ) -> None:
         """Process a change in typing of a room from replication, sending EDUs
         for any local users.
@@ -263,6 +266,7 @@ class TypingWriterHandler(FollowerTypingHandler):
 
         assert hs.get_instance_name() in hs.config.worker.writers.typing
 
+        self.server_name = hs.hostname
         self.auth = hs.get_auth()
         self.notifier = hs.get_notifier()
         self.event_auth_handler = hs.get_event_auth_handler()
@@ -276,11 +280,13 @@ class TypingWriterHandler(FollowerTypingHandler):
         hs.get_distributor().observe("user_left_room", self.user_left_room)
 
         # clock time we expect to stop
-        self._member_typing_until: Dict[RoomMember, int] = {}
+        self._member_typing_until: dict[RoomMember, int] = {}
 
         # caches which room_ids changed at which serials
         self._typing_stream_change_cache = StreamChangeCache(
-            "TypingStreamChangeCache", self._latest_room_serial
+            name="TypingStreamChangeCache",
+            server_name=self.server_name,
+            current_stream_pos=self._latest_room_serial,
         )
 
     def _handle_timeout_for_member(self, now: int, member: RoomMember) -> None:
@@ -374,8 +380,11 @@ class TypingWriterHandler(FollowerTypingHandler):
     def _push_update(self, member: RoomMember, typing: bool) -> None:
         if self.hs.is_mine_id(member.user_id):
             # Only send updates for changes to our own users.
-            run_as_background_process(
-                "typing._push_remote", self._push_remote, member, typing
+            self.hs.run_as_background_process(
+                "typing._push_remote",
+                self._push_remote,
+                member,
+                typing,
             )
 
         self._push_update_local(member=member, typing=typing)
@@ -443,7 +452,7 @@ class TypingWriterHandler(FollowerTypingHandler):
 
     async def get_all_typing_updates(
         self, instance_name: str, last_id: int, current_id: int, limit: int
-    ) -> Tuple[List[Tuple[int, list]], int, bool]:
+    ) -> tuple[list[tuple[int, list]], int, bool]:
         """Get updates for typing replication stream.
 
         Args:
@@ -495,7 +504,7 @@ class TypingWriterHandler(FollowerTypingHandler):
         return rows, current_id, limited
 
     def process_replication_rows(
-        self, token: int, rows: List[TypingStream.TypingStreamRow]
+        self, token: int, rows: list[TypingStream.TypingStreamRow]
     ) -> None:
         # The writing process should never get updates from replication.
         raise Exception("Typing writer instance got typing info over replication")
@@ -503,6 +512,7 @@ class TypingWriterHandler(FollowerTypingHandler):
 
 class TypingNotificationEventSource(EventSource[int, JsonMapping]):
     def __init__(self, hs: "HomeServer"):
+        self.server_name = hs.hostname
         self._main_store = hs.get_datastores().main
         self.clock = hs.get_clock()
         # We can't call get_typing_handler here because there's a cycle:
@@ -521,7 +531,7 @@ class TypingNotificationEventSource(EventSource[int, JsonMapping]):
 
     async def get_new_events_as(
         self, from_key: int, service: ApplicationService
-    ) -> Tuple[List[JsonMapping], int]:
+    ) -> tuple[list[JsonMapping], int]:
         """Returns a set of new typing events that an appservice
         may be interested in.
 
@@ -535,7 +545,9 @@ class TypingNotificationEventSource(EventSource[int, JsonMapping]):
                   appservice may be interested in.
                 * The latest known room serial.
         """
-        with Measure(self.clock, "typing.get_new_events_as"):
+        with Measure(
+            self.clock, name="typing.get_new_events_as", server_name=self.server_name
+        ):
             handler = self.get_typing_handler()
 
             events = []
@@ -565,8 +577,15 @@ class TypingNotificationEventSource(EventSource[int, JsonMapping]):
         room_ids: Iterable[str],
         is_guest: bool,
         explicit_room_id: Optional[str] = None,
-    ) -> Tuple[List[JsonMapping], int]:
-        with Measure(self.clock, "typing.get_new_events"):
+        to_key: Optional[int] = None,
+    ) -> tuple[list[JsonMapping], int]:
+        """
+        Find typing notifications for given rooms (> `from_token` and <= `to_token`)
+        """
+
+        with Measure(
+            self.clock, name="typing.get_new_events", server_name=self.server_name
+        ):
             from_key = int(from_key)
             handler = self.get_typing_handler()
 
@@ -574,7 +593,9 @@ class TypingNotificationEventSource(EventSource[int, JsonMapping]):
             for room_id in room_ids:
                 if room_id not in handler._room_serials:
                     continue
-                if handler._room_serials[room_id] <= from_key:
+                if handler._room_serials[room_id] <= from_key or (
+                    to_key is not None and handler._room_serials[room_id] > to_key
+                ):
                     continue
 
                 events.append(self._make_event_for(room_id))
