@@ -26,9 +26,7 @@ from typing import (
     Any,
     Awaitable,
     Callable,
-    List,
     Optional,
-    Tuple,
     TypeVar,
 )
 
@@ -56,7 +54,7 @@ Token = int
 # parsing with Stream.parse_row (which turns it into a `ROW_TYPE`). Normally it's
 # just a row from a database query, though this is dependent on the stream in question.
 #
-StreamRow = TypeVar("StreamRow", bound=Tuple)
+StreamRow = TypeVar("StreamRow", bound=tuple)
 
 # The type returned by the update_function of a stream, as well as get_updates(),
 # get_updates_since, etc.
@@ -66,7 +64,7 @@ StreamRow = TypeVar("StreamRow", bound=Tuple)
 #   * `new_last_token` is the new position in stream.
 #   * `limited` is whether there are more updates to fetch.
 #
-StreamUpdateResult = Tuple[List[Tuple[Token, StreamRow]], Token, bool]
+StreamUpdateResult = tuple[list[tuple[Token, StreamRow]], Token, bool]
 
 # The type of an update_function for a stream
 #
@@ -400,7 +398,7 @@ class TypingStream(Stream):
         room_id: str
 
         # All the users that are 'typing' right now in the specified room.
-        user_ids: List[str]
+        user_ids: list[str]
 
     NAME = "typing"
     ROW_TYPE = TypingStreamRow
@@ -410,7 +408,7 @@ class TypingStream(Stream):
             # On the writer, query the typing handler
             typing_writer_handler = hs.get_typing_writer_handler()
             update_function: Callable[
-                [str, int, int, int], Awaitable[Tuple[List[Tuple[int, Any]], int, bool]]
+                [str, int, int, int], Awaitable[tuple[list[tuple[int, Any]], int, bool]]
             ] = typing_writer_handler.get_all_typing_updates
             self.current_token_function = typing_writer_handler.get_current_token
         else:
@@ -512,7 +510,7 @@ class CachesStream(Stream):
         """
 
         cache_func: str
-        keys: Optional[List[Any]]
+        keys: Optional[list[Any]]
         invalidation_ts: int
 
     NAME = "caches"
@@ -739,7 +737,7 @@ class ThreadSubscriptionsStream(_StreamFromIdGen):
     NAME = "thread_subscriptions"
     ROW_TYPE = ThreadSubscriptionsStreamRow
 
-    def __init__(self, hs: Any):
+    def __init__(self, hs: "HomeServer"):
         self.store = hs.get_datastores().main
         super().__init__(
             hs.get_instance_name(),
@@ -751,7 +749,7 @@ class ThreadSubscriptionsStream(_StreamFromIdGen):
         self, instance_name: str, from_token: int, to_token: int, limit: int
     ) -> StreamUpdateResult:
         updates = await self.store.get_updated_thread_subscriptions(
-            from_token, to_token, limit
+            from_id=from_token, to_id=to_token, limit=limit
         )
         rows = [
             (
