@@ -28,10 +28,7 @@ from typing import (
     Any,
     Collection,
     Iterable,
-    List,
     Optional,
-    Set,
-    Tuple,
     Union,
     cast,
 )
@@ -362,7 +359,7 @@ class SearchBackgroundUpdateStore(SearchWorkerStore):
                 pg,
             )
 
-        def reindex_search_txn(txn: LoggingTransaction) -> Tuple[int, bool]:
+        def reindex_search_txn(txn: LoggingTransaction) -> tuple[int, bool]:
             sql = """
             UPDATE event_search AS es
             SET stream_ordering = e.stream_ordering, origin_server_ts = e.origin_server_ts
@@ -451,7 +448,7 @@ class SearchStore(SearchBackgroundUpdateStore):
         """
         clauses = []
 
-        args: List[Any] = []
+        args: list[Any] = []
 
         # Make sure we don't explode because the person is in too many rooms.
         # We filter the results below regardless.
@@ -471,7 +468,7 @@ class SearchStore(SearchBackgroundUpdateStore):
         count_args = args
         count_clauses = clauses
 
-        sqlite_highlights: List[str] = []
+        sqlite_highlights: list[str] = []
 
         if isinstance(self.database_engine, PostgresEngine):
             search_query = search_term
@@ -519,7 +516,7 @@ class SearchStore(SearchBackgroundUpdateStore):
 
         # List of tuples of (rank, room_id, event_id).
         results = cast(
-            List[Tuple[Union[int, float], str, str]],
+            list[tuple[Union[int, float], str, str]],
             await self.db_pool.execute("search_msgs", sql, *args),
         )
 
@@ -544,7 +541,7 @@ class SearchStore(SearchBackgroundUpdateStore):
 
         # List of tuples of (room_id, count).
         count_results = cast(
-            List[Tuple[str, int]],
+            list[tuple[str, int]],
             await self.db_pool.execute("search_rooms_count", count_sql, *count_args),
         )
 
@@ -580,7 +577,7 @@ class SearchStore(SearchBackgroundUpdateStore):
             Each match as a dictionary.
         """
         clauses = []
-        args: List[Any] = []
+        args: list[Any] = []
 
         # Make sure we don't explode because the person is in too many rooms.
         # We filter the results below regardless.
@@ -602,7 +599,7 @@ class SearchStore(SearchBackgroundUpdateStore):
         count_args = list(args)
         count_clauses = list(clauses)
 
-        sqlite_highlights: List[str] = []
+        sqlite_highlights: list[str] = []
 
         if pagination_token:
             try:
@@ -686,7 +683,7 @@ class SearchStore(SearchBackgroundUpdateStore):
 
         # List of tuples of (rank, room_id, event_id, origin_server_ts, stream_ordering).
         results = cast(
-            List[Tuple[Union[int, float], str, str, int, int]],
+            list[tuple[Union[int, float], str, str, int, int]],
             await self.db_pool.execute("search_rooms", sql, *args),
         )
 
@@ -711,7 +708,7 @@ class SearchStore(SearchBackgroundUpdateStore):
 
         # List of tuples of (room_id, count).
         count_results = cast(
-            List[Tuple[str, int]],
+            list[tuple[str, int]],
             await self.db_pool.execute("search_rooms_count", count_sql, *count_args),
         )
 
@@ -732,8 +729,8 @@ class SearchStore(SearchBackgroundUpdateStore):
         }
 
     async def _find_highlights_in_postgres(
-        self, search_query: str, events: List[EventBase]
-    ) -> Set[str]:
+        self, search_query: str, events: list[EventBase]
+    ) -> set[str]:
         """Given a list of events and a search term, return a list of words
         that match from the content of the event.
 
@@ -748,7 +745,7 @@ class SearchStore(SearchBackgroundUpdateStore):
             A set of strings.
         """
 
-        def f(txn: LoggingTransaction) -> Set[str]:
+        def f(txn: LoggingTransaction) -> set[str]:
             highlight_words = set()
             for event in events:
                 # As a hack we simply join values of all possible keys. This is
@@ -811,7 +808,7 @@ def _to_postgres_options(options_dict: JsonDict) -> str:
 
 @dataclass
 class Phrase:
-    phrase: List[str]
+    phrase: list[str]
 
 
 class SearchToken(enum.Enum):
@@ -821,7 +818,7 @@ class SearchToken(enum.Enum):
 
 
 Token = Union[str, Phrase, SearchToken]
-TokenList = List[Token]
+TokenList = list[Token]
 
 
 def _is_stop_word(word: str) -> bool:
@@ -901,7 +898,7 @@ def _tokenize_query(query: str) -> TokenList:
     return tokens
 
 
-def _tokens_to_sqlite_match_query(tokens: TokenList) -> Tuple[str, List[str]]:
+def _tokens_to_sqlite_match_query(tokens: TokenList) -> tuple[str, list[str]]:
     """
     Convert the list of tokens to a string suitable for passing to sqlite's MATCH.
     Assume sqlite was compiled with enhanced query syntax.
@@ -934,7 +931,7 @@ def _tokens_to_sqlite_match_query(tokens: TokenList) -> Tuple[str, List[str]]:
     return "".join(match_query), highlights
 
 
-def _parse_query_for_sqlite(search_term: str) -> Tuple[str, List[str]]:
+def _parse_query_for_sqlite(search_term: str) -> tuple[str, list[str]]:
     """Takes a plain unicode string from the user and converts it into a form
     that can be passed to sqllite's matchinfo().
 
