@@ -21,12 +21,9 @@ from typing import (
     Collection,
     Mapping,
     Optional,
-    Set,
-    Tuple,
 )
 
-from synapse.events import EventBase
-from synapse.events.snapshot import EventContext
+from synapse.events.snapshot import EventPersistencePair
 from synapse.storage.database import (
     DatabasePool,
     LoggingDatabaseConnection,
@@ -228,13 +225,13 @@ class StateDeletionDataStore:
 
     @contextlib.asynccontextmanager
     async def persisting_state_group_references(
-        self, event_and_contexts: Collection[Tuple[EventBase, EventContext]]
+        self, event_and_contexts: Collection[EventPersistencePair]
     ) -> AsyncIterator[None]:
         """Wraps the persistence of the given events and contexts, ensuring that
         any state groups referenced still exist and that they don't get deleted
         during this."""
 
-        referenced_state_groups: Set[int] = set()
+        referenced_state_groups: set[int] = set()
         for event, ctx in event_and_contexts:
             if ctx.rejected or event.internal_metadata.is_outlier():
                 continue
@@ -270,7 +267,7 @@ class StateDeletionDataStore:
             )
 
     def _mark_state_groups_as_persisting_txn(
-        self, txn: LoggingTransaction, state_groups: Set[int]
+        self, txn: LoggingTransaction, state_groups: set[int]
     ) -> None:
         """Marks the given state groups as being persisted."""
 
@@ -509,7 +506,7 @@ class StateDeletionDataStore:
 
     async def get_next_state_group_collection_to_delete(
         self,
-    ) -> Optional[Tuple[str, Mapping[int, int]]]:
+    ) -> Optional[tuple[str, Mapping[int, int]]]:
         """Get the next set of state groups to try and delete
 
         Returns:
@@ -523,7 +520,7 @@ class StateDeletionDataStore:
     def _get_next_state_group_collection_to_delete_txn(
         self,
         txn: LoggingTransaction,
-    ) -> Optional[Tuple[str, Mapping[int, int]]]:
+    ) -> Optional[tuple[str, Mapping[int, int]]]:
         """Implementation of `get_next_state_group_collection_to_delete`"""
 
         # We want to return chunks of state groups that were marked for deletion
